searchNodes=[{"ref":"Brex.Result.html","title":"Brex.Result","module":"Brex.Result","type":"module","doc":"This library provides tools to handle three common return values in Elixir :ok | {:ok, value} | {:error, reason} Brex.Result is split into three main components: Brex.Result.Base - Base provides tools for creating and passing around ok/error tuples. The tools follow the property: if there’s a success continue the computation, if there’s an error propagate it. Brex.Result.Helpers - Helpers includes tools for modifying the reason in error tuples. The functions in this module always propogate the success value. Brex.Result.Mappers - Mappers includes tools for applying functions that return :ok | {:ok, val} | {:error, reason} over Enumerables. To import the entire library: use Brex.Result To import the modules individually: import Brex.Result.Base import Brex.Result.Helpers import Brex.Result.Mappers"},{"ref":"Brex.Result.html#t:p/0","title":"Brex.Result.p/0","module":"Brex.Result","type":"type","doc":":ok | {:error, any}"},{"ref":"Brex.Result.html#t:s/0","title":"Brex.Result.s/0","module":"Brex.Result","type":"type","doc":"{:ok, any} | {:error, any}"},{"ref":"Brex.Result.html#t:s/1","title":"Brex.Result.s/1","module":"Brex.Result","type":"type","doc":"{:ok, x} | {:error, any}"},{"ref":"Brex.Result.html#t:t/0","title":"Brex.Result.t/0","module":"Brex.Result","type":"type","doc":":ok | {:ok, any} | {:error, any}"},{"ref":"Brex.Result.html#t:t/1","title":"Brex.Result.t/1","module":"Brex.Result","type":"type","doc":":ok | {:ok, x} | {:error, any}"},{"ref":"Brex.Result.Base.html","title":"Brex.Result.Base","module":"Brex.Result.Base","type":"module","doc":"Tools for doing basic result tuple manipulations."},{"ref":"Brex.Result.Base.html#bind/2","title":"Brex.Result.Base.bind/2","module":"Brex.Result.Base","type":"function","doc":"Takes in a tuple and function from plain value to {:ok, any} | {:error, any}. Applies the function to the value within the ok tuple or propogates the error. Examples: iex&gt; bind({:ok, 1}, fn x -&gt; if x == 1, do: {:ok, 2}, else: {:error, &quot;not_one&quot;} end) {:ok, 2} iex&gt; bind({:ok, 4}, fn x -&gt; if x == 1, do: {:ok, 2}, else: {:error, &quot;not_one&quot;} end) {:error, &quot;not_one&quot;} iex&gt; bind({:error, 4}, fn x -&gt; if x == 1, do: {:ok, 2}, else: {:error, &quot;not_one&quot;} end) {:error, 4}"},{"ref":"Brex.Result.Base.html#error/1","title":"Brex.Result.Base.error/1","module":"Brex.Result.Base","type":"macro","doc":"Wraps value in an error tuple Will be inlined at compile time. Typespec: error(any) :: t()"},{"ref":"Brex.Result.Base.html#extract!/1","title":"Brex.Result.Base.extract!/1","module":"Brex.Result.Base","type":"function","doc":"Extracts the value or reason from the tuple. Caution: If given an error tuple it raise an exception!"},{"ref":"Brex.Result.Base.html#fmap/2","title":"Brex.Result.Base.fmap/2","module":"Brex.Result.Base","type":"function","doc":"Takes in a tuple and a function from plain value to plain value. Applies the function to the value within the ok tuple or propogates error. Examples: iex&gt; {:ok, 6} ...&gt; |&gt; fmap(fn x -&gt; x+2 end) {:ok, 8} iex&gt; {:error, 6} ...&gt; |&gt; fmap(fn x -&gt; x+2 end) {:error, 6}"},{"ref":"Brex.Result.Base.html#ignore/1","title":"Brex.Result.Base.ignore/1","module":"Brex.Result.Base","type":"function","doc":"Ignores the value in an ok tuple and just returns :ok. Still shortcircuits on error. Examples: iex&gt; {:ok, 2} ...&gt; |&gt; ignore :ok iex&gt; :ok ...&gt; |&gt; ignore :ok iex&gt; {:error, :not_found} ...&gt; |&gt; ignore {:error, :not_found}"},{"ref":"Brex.Result.Base.html#ok/1","title":"Brex.Result.Base.ok/1","module":"Brex.Result.Base","type":"macro","doc":"Wraps value in an ok tuple. Will be inlined at compile time. Typespec: ok(a) :: s(a) when a: var"},{"ref":"Brex.Result.Base.html#~>/2","title":"Brex.Result.Base.~>/2","module":"Brex.Result.Base","type":"macro","doc":"This is infix bind/2 Has same syntax restrictions as pipe. Defined as a macro for syntatic purposes. Examples: def sgn(x) do if x &gt; 0 do {:ok, &quot;pos&quot;} else {:error, &quot;neg&quot;} end end def two_args(x, y), do: {:ok, x - y} {:ok, 1} ~&gt; sgn = {:ok, &quot;pos&quot;} {:ok, -3} ~&gt; sgn = {:error, &quot;neg&quot;} {:error, 2} ~&gt; sgn = {:error, 2} {:ok, 3} ~&gt; two_args(2) = {:ok, 1} Typespec: t(a) ~&gt; (a -&gt; t(b)) :: t(b) when a: var, b: var"},{"ref":"Brex.Result.Base.html#t:p/0","title":"Brex.Result.Base.p/0","module":"Brex.Result.Base","type":"type","doc":""},{"ref":"Brex.Result.Base.html#t:s/0","title":"Brex.Result.Base.s/0","module":"Brex.Result.Base","type":"type","doc":""},{"ref":"Brex.Result.Base.html#t:s/1","title":"Brex.Result.Base.s/1","module":"Brex.Result.Base","type":"type","doc":""},{"ref":"Brex.Result.Base.html#t:t/0","title":"Brex.Result.Base.t/0","module":"Brex.Result.Base","type":"type","doc":""},{"ref":"Brex.Result.Base.html#t:t/1","title":"Brex.Result.Base.t/1","module":"Brex.Result.Base","type":"type","doc":""},{"ref":"Brex.Result.Helpers.html","title":"Brex.Result.Helpers","module":"Brex.Result.Helpers","type":"module","doc":"Tools for modifying the reason in error tuples."},{"ref":"Brex.Result.Helpers.html#convert_error/2","title":"Brex.Result.Helpers.convert_error/2","module":"Brex.Result.Helpers","type":"function","doc":"Converts a matching error to :ok An error matches if the reason is equal to the supplied atom or the reason passes the predicate. Leaves success and other errors unchanged. Examples: iex&gt; {:error, :already_completed} ...&gt; |&gt; convert_error(:already_completed) :ok iex&gt; {:error, :already_completed} ...&gt; |&gt; convert_error(fn r -&gt; r == :already_completed end) :ok"},{"ref":"Brex.Result.Helpers.html#convert_error/3","title":"Brex.Result.Helpers.convert_error/3","module":"Brex.Result.Helpers","type":"macro","doc":"Converts a matching error to a success with the given value or function. An error matches if the reason is equal to the supplied atom or the reason passes the predicate. Leaves success and other errors unchanged. Lazy. Only evaluates the second argument if necessary. Examples: iex&gt; {:error, :already_completed} ...&gt; |&gt; convert_error(:already_completed, &quot;submitted&quot;) {:ok, &quot;submitted&quot;} iex&gt; {:error, &quot;test&quot;} ...&gt; |&gt; convert_error(&amp;(&amp;1 == &quot;test&quot;), fn r -&gt; {:ok, r &lt;&gt; &quot;ed&quot;} end) {:ok, &quot;tested&quot;} Typespec: convert_error(t(), (any -&gt; boolean) | any, b | (any -&gt; t(b))) :: t(b) when b: var"},{"ref":"Brex.Result.Helpers.html#lift/3","title":"Brex.Result.Helpers.lift/3","module":"Brex.Result.Helpers","type":"macro","doc":"Lifts a value into a success tuple unless: 1) the value matches the second argument 2) when applied to the value, the second argument function returns true In those cases an error tuple is returned with either 1) the third argument as the reason 2) the third argument function applied to the value, as the reason lift/3 is lazy, this means third argument will only be evaluated when necessary. Examples: iex&gt; nil ...&gt; |&gt; lift(nil, :not_found) {:error, :not_found} iex&gt; 2 ...&gt; |&gt; lift(nil, :not_found) {:ok, 2} iex&gt; &quot;test&quot; ...&gt; |&gt; lift(&amp;(&amp;1 == &quot;test&quot;), fn x -&gt; {:oops, x &lt;&gt; &quot;ed&quot;} end) {:error, {:oops, &quot;tested&quot;}} Typespec: lift(a | b, b | (a | b -&gt; boolean), c | (a | b -&gt; c)) :: s(a) when a: var, b: var, c: var"},{"ref":"Brex.Result.Helpers.html#log_error/3","title":"Brex.Result.Helpers.log_error/3","module":"Brex.Result.Helpers","type":"function","doc":"Logs on error, does nothing on success. Example: {:error, :not_found} |&gt; log_error(&quot;There was an error&quot;, level: :info, metadata: &quot;some meta&quot;)"},{"ref":"Brex.Result.Helpers.html#map_error/2","title":"Brex.Result.Helpers.map_error/2","module":"Brex.Result.Helpers","type":"function","doc":"Applies the function to the reason in an error tuple. Leaves success unchanged. Example: iex&gt; account_name = &quot;test&quot; ...&gt; {:error, :not_found} ...&gt; |&gt; map_error(fn r -&gt; {r, account_name} end) {:error, {:not_found, &quot;test&quot;}}"},{"ref":"Brex.Result.Helpers.html#mask_error/2","title":"Brex.Result.Helpers.mask_error/2","module":"Brex.Result.Helpers","type":"macro","doc":"Replaces the reason in an error tuple. Leaves success unchanged. Lazy. Only evaluates the second argument if necessary. Example: iex&gt; account_name = &quot;test&quot; ...&gt; {:error, :not_found} ...&gt; |&gt; mask_error({:nonexistent_account, account_name}) {:error, {:nonexistent_account, &quot;test&quot;}} Typespec: mask_error(t(a), any) :: t(a) when a: var"},{"ref":"Brex.Result.Helpers.html#normalize_error/2","title":"Brex.Result.Helpers.normalize_error/2","module":"Brex.Result.Helpers","type":"function","doc":"Wraps a naked :error atom in a tuple with the given reason. Leaves success values and error tuples unchanged. Examples: iex&gt; :error ...&gt; |&gt; normalize_error(:parsing_failure) {:error, :parsing_failure} iex&gt; {:ok, 2} ...&gt; |&gt; normalize_error() {:ok, 2}"},{"ref":"Brex.Result.Helpers.html#t:p/0","title":"Brex.Result.Helpers.p/0","module":"Brex.Result.Helpers","type":"type","doc":""},{"ref":"Brex.Result.Helpers.html#t:s/0","title":"Brex.Result.Helpers.s/0","module":"Brex.Result.Helpers","type":"type","doc":""},{"ref":"Brex.Result.Helpers.html#t:s/1","title":"Brex.Result.Helpers.s/1","module":"Brex.Result.Helpers","type":"type","doc":""},{"ref":"Brex.Result.Helpers.html#t:t/0","title":"Brex.Result.Helpers.t/0","module":"Brex.Result.Helpers","type":"type","doc":""},{"ref":"Brex.Result.Helpers.html#t:t/1","title":"Brex.Result.Helpers.t/1","module":"Brex.Result.Helpers","type":"type","doc":""},{"ref":"Brex.Result.Mappers.html","title":"Brex.Result.Mappers","module":"Brex.Result.Mappers","type":"module","doc":"Tools for combining Enum and result tuples."},{"ref":"Brex.Result.Mappers.html#each_while_success/2","title":"Brex.Result.Mappers.each_while_success/2","module":"Brex.Result.Mappers","type":"function","doc":"Applies the function to each element of the enumerable until an error occurs. No guarentee on the order of evaluation. (But usually backwards for lists.) Only takes a function that returns :ok | {:error, value}. Examples: iex&gt; [1, 2, 3, 4] ...&gt; |&gt; each_while_success(fn x -&gt; if x &lt; 3, do: :ok, else: {:error, :too_big} end) {:error, :too_big} iex&gt; [1, 2, 3, 4] ...&gt; |&gt; each_while_success(fn x -&gt; if x &lt; 5, do: :ok, else: {:error, :too_big} end) :ok"},{"ref":"Brex.Result.Mappers.html#map_while_success/2","title":"Brex.Result.Mappers.map_while_success/2","module":"Brex.Result.Mappers","type":"function","doc":"Given an enumerable of plain values, it returns {:ok, processed enum} or the first error. Equivalent to traverse or mapM in Haskell. Examples: iex&gt; [1, 2, 3, 4] ...&gt; |&gt; map_while_success(fn x -&gt; if x == 3 || x == 1, do: {:error, x}, else: {:ok, x} end) {:error, 1} iex&gt; map_while_success([1, 2, 3, 4], fn x -&gt; {:ok, x + 2} end) {:ok, [3, 4, 5, 6]}"},{"ref":"Brex.Result.Mappers.html#map_with_bind/2","title":"Brex.Result.Mappers.map_with_bind/2","module":"Brex.Result.Mappers","type":"function","doc":"Binds the function to each tuple in the enum. Example: iex&gt; [{:ok, 1}, {:ok, 2}, {:error, 3}, {:ok, 4}] ...&gt; |&gt; map_with_bind(fn x -&gt; if x == 2, do: {:error, x*6}, else: {:ok, x*6} end) [{:ok, 6}, {:error, 12}, {:error, 3}, {:ok, 24}]"},{"ref":"Brex.Result.Mappers.html#reduce_while_success/3","title":"Brex.Result.Mappers.reduce_while_success/3","module":"Brex.Result.Mappers","type":"function","doc":"Given an enum of plain values, an initial value, and a reducing function, it returns the first error or reduced result. Examples: iex&gt; [1, 2, 3, 4] ...&gt; |&gt; reduce_while_success(100, &amp;{:ok, &amp;1 + &amp;2}) {:ok, 110} iex&gt; [1, 2, 3, 4] ...&gt; |&gt; reduce_while_success(100, fn x, acc -&gt; ...&gt; if x &gt; 2 do ...&gt; {:error, x} ...&gt; else ...&gt; {:ok, x + acc} ...&gt; end ...&gt; end) {:error, 3}"},{"ref":"Brex.Result.Mappers.html#t:enumerable/1","title":"Brex.Result.Mappers.enumerable/1","module":"Brex.Result.Mappers","type":"type","doc":"The type Enum.t/0 does not accept any arguments. This is a workaround to express the type of an enumerable with elements restricted to a particular type."},{"ref":"Brex.Result.Mappers.html#t:p/0","title":"Brex.Result.Mappers.p/0","module":"Brex.Result.Mappers","type":"type","doc":""},{"ref":"Brex.Result.Mappers.html#t:s/0","title":"Brex.Result.Mappers.s/0","module":"Brex.Result.Mappers","type":"type","doc":""},{"ref":"Brex.Result.Mappers.html#t:s/1","title":"Brex.Result.Mappers.s/1","module":"Brex.Result.Mappers","type":"type","doc":""},{"ref":"Brex.Result.Mappers.html#t:t/0","title":"Brex.Result.Mappers.t/0","module":"Brex.Result.Mappers","type":"type","doc":""},{"ref":"Brex.Result.Mappers.html#t:t/1","title":"Brex.Result.Mappers.t/1","module":"Brex.Result.Mappers","type":"type","doc":""},{"ref":"readme.html","title":"Brex.Result","module":"readme","type":"extras","doc":"Brex.Result This library provides tools to handle three common return values in Elixir :ok | {:ok, value} | {:error, reason}"},{"ref":"readme.html#table-of-contents","title":"Brex.Result - Table of Contents","module":"readme","type":"extras","doc":"Overview Usage Differences rom Similar Libraries Definitions Types Base Helpers Mappers Known Problems Installation"},{"ref":"readme.html#overview","title":"Brex.Result - Overview","module":"readme","type":"extras","doc":"Brex.Result is split into three main components: Brex.Result.Base - Base provides tools for creating and passing around ok/error tuples. The tools follow the property: if there’s a success continue the computation, if there’s an error propagate it. Brex.Result.Helpers - Helpers includes tools for modifying the reason in error tuples. The functions in this module always propogate the success value. Brex.Result.Mappers - Mappers includes tools for applying functions that return :ok | {:ok, val} | {:error, reason} over Enumerables."},{"ref":"readme.html#usage","title":"Brex.Result - Usage","module":"readme","type":"extras","doc":"Include the line use Brex.Result to import the entire library or import Brex.Result.{Base, Helpers, Mappers} to import the modules individually. A sampling of functions and examples are provided below. For more in-depth examples see examples."},{"ref":"readme.html#differences-from-similar-libraries","title":"Brex.Result - Differences from Similar Libraries","module":"readme","type":"extras","doc":"Other libraries like OK, Monad, and Ok Jose have embraced the concept of monadic error handling and have analogous functions to the ones we have in Brex.Result.Base. Brex.Result separates itself by: Extending support beyond classic monadic functions support for :ok as a success value support for modifying errors tuple reasons support for mapping functions that return ok | {:ok, value} | {:error, reason} over enumerables Respecting Elixir idioms encourages use of elixir builtins like the with statement when appropriate provides style guidelines actively avoids heavy macro magic that can turn a library into a DSL"},{"ref":"readme.html#definitions","title":"Brex.Result - Definitions","module":"readme","type":"extras","doc":"Result tuples:{:ok, value} | {:error, reason} Error tuples: {:error, reason} OK/Success tuples: {:ok, value} Success values: :ok | {:ok, value} Propagate a value: Return a value unchanged"},{"ref":"readme.html#types","title":"Brex.Result - Types","module":"readme","type":"extras","doc":"Parametrized types @type s(x) :: {:ok, x} | {:error, any} @type t(x) :: :ok | s(x) Convenience types @type p() :: :ok | {:error, any} @type s() :: s(any) @type t() :: t(any) Style Recommendation Write specs and callbacks usings these shorthands. # Original @spec my_fun({:ok, String.t} | {:error, any}, Integer) :: :ok | {:error, any} # With shorthands @spec my_fun(Brex.Result.s(String.t), Integer) :: Brex.Result.p()"},{"ref":"readme.html#base","title":"Brex.Result - Base","module":"readme","type":"extras","doc":"Use Brex.Result.Base.ok/1 to wrap a value in an ok tuple. iex&gt; 2 ...&gt; |&gt; ok {:ok, 2} Brex.Result.Base.error/1 wraps a value in an error tuple. iex&gt; :not_found ...&gt; |&gt; error {:error, :not_found} Style Recommendation Don&#39;t use ok/1 and error/1 when tuple syntax is more explicit: # No ok(2) # Yes {:ok, 2} Do use ok/1 and error/1 at the end of a chain of pipes: # No val = arg |&gt; get_values() |&gt; transform(other_arg) {:ok, val} # Yes arg |&gt; get_values() |&gt; transform(other_arg) |&gt; ok Use Brex.Result.Base.fmap/2 to transform the value within a success tuple. It propogates the error value. iex&gt; {:ok, 2} ...&gt; |&gt; fmap(fn x -&gt; x + 5 end) {:ok, 7} iex&gt; {:error, :not_found} ...&gt; |&gt; fmap(fn x -&gt; x + 5 end) {:error, :not_found} Use Brex.Result.Base.bind/2 to apply a function to the value within a success tuple. The function must returns a result tuple. iex&gt; {:ok, 2} ...&gt; |&gt; bind(fn x -&gt; if x &gt; 0, do: {:ok, x + 5}, else: {:error, :neg}) {:ok, 7} iex&gt; {:ok, -1} ...&gt; |&gt; bind(fn x -&gt; if x &gt; 0, do: {:ok, x + 5}, else: {:error, :neg}) {:error, :neg} iex&gt; {:error, :not_found} ...&gt; |&gt; bind(fn x -&gt; if x &gt; 0, do: {:ok, x + 5}, else: {:error, :neg}) {:error, :not_found} Infix bind is given for convience as Brex.Result.Base.~&gt;/2 iex&gt; {:ok, [[1, 2, 3, 4]} ...&gt; ~&gt; Enum.member(2) ...&gt; |&gt; fmap(fn x -&gt; if x, do: :found_two, else: :not_found end) {:ok, :found_two} Style Recommendation Avoid single ~&gt;s and only use ~&gt; when the function argument is named and fits onto one line. # No {:ok, file} ~&gt; File.read() # Yes bind({:ok, file}, &amp;File.read/1) # No {:ok, val} ~&gt; (fn x -&gt; if x &gt; 0, do: {:ok, x}, else: {:error, neg}).() ~&gt; insert_amount() # Yes {:ok, val} |&gt; bind(fn x -&gt; if x &gt; 0, do: {:ok, x}, else: {:error, neg}) ~&gt; insert_amount()"},{"ref":"readme.html#helpers","title":"Brex.Result - Helpers","module":"readme","type":"extras","doc":"Brex.Result.Helpers.map_error/2 allows you to transform the reason in an error tuple. iex&gt; {:error, 404} ...&gt; |&gt; map_error(fn reason -&gt; {:invalid_response, reason} end) {:error, {:invalid_reponse, 404}} iex&gt; {:ok, 2} ...&gt; |&gt; map_error(fn reason -&gt; {:invalid_response, reason} end) {:ok, 2} iex&gt; :ok ...&gt; |&gt; map_error(fn reason -&gt; {:invalid_response, reason} end) :ok Brex.Result.Helpers.mask_error/2 disregards the current reason and replaces it with the second argument. iex&gt; {:error, :not_found} ...&gt; |&gt; mask_error(:failure) {:error, :failure} Brex.Result.Helpers.convert_error/3 converts an error into a success value if the reason matches the second argument. iex&gt; {:error, :not_found} ...&gt; |&gt; convert_error(:not_found) :ok iex&gt; {:error, :not_found} ...&gt; |&gt; convert_error(:not_found, default) {:ok, default} Brex.Result.Helpers.log_error/3 logs on error. It automatically includes the reason in the log metadata. iex&gt; {:error, :not_found} ...&gt; |&gt; log_error(&quot;There was a problem&quot;, level: :warn) {:error, :not_found} Brex.Result.Helpers.normalize_error/2 converts a naked :error atom into an error tuple. It&#39;s good for functions from outside libraries. iex&gt; :error ...&gt; |&gt; normalize_error(:not_found) {:error, :not_found} iex&gt; {:ok, 2} ...&gt; |&gt; normalize_error(:not_found) {:ok, 2} iex&gt; :ok ...&gt; |&gt; normalize_error(:not_found) :ok"},{"ref":"readme.html#mappers","title":"Brex.Result - Mappers","module":"readme","type":"extras","doc":"Brex.Result.Mappers.map_while_success/2, Brex.Result.Mappers.each_while_success/2, Brex.Result.Mappers.reduce_while_success/3 all mimic the Enum functions Enum.map/2, Enum.each/2, Enum.reduce/3, but take a function that returns :ok | {:ok, value} | {:error, reason} as the mapping/reducing argument. Each of these functions produce a success value containing the final result or the first error."},{"ref":"readme.html#known-problems","title":"Brex.Result - Known Problems","module":"readme","type":"extras","doc":"Credo complains pipe chain is not started with raw value when preceeded by ~&gt;."},{"ref":"readme.html#installation","title":"Brex.Result - Installation","module":"readme","type":"extras","doc":"The package can be installed by adding brex_result to your list of dependencies in mix.exs: def deps do [ {:brex_result, &quot;~&gt; 0.4.0&quot;} ] end"},{"ref":"contributing.html","title":"Contributing","module":"contributing","type":"extras","doc":"Contributing Fork it! Create your feature branch (git checkout -b my-new-feature) Commit your changes (git commit -am &#39;Add some feature&#39;) Push to the branch (git push origin my-new-feature) Create new Pull Request"},{"ref":"changelog.html","title":"Changelog","module":"changelog","type":"extras","doc":"Changelog"},{"ref":"changelog.html#v0-4-0-04-10-19","title":"Changelog - v0.4.0 04/10/19","module":"changelog","type":"extras","doc":"Removed @doc &quot;&quot;&quot; A dangerous pattern matching gaurd. on_match(expr, pattern, f) is equivalent to: case expr of pattern -&gt; f.(expr) _ -&gt; expr ## Examples: iex&gt; {:bad, 1} ...&gt; |&gt; on_match({:good, _}, fn x -&gt; x + 2 end) {:bad, 1} iex&gt; {:good, 1} ...&gt; |&gt; on_match({:good, _}, fn {:good, x} -&gt; x + 2 end) 3 &quot;&quot;&quot; @spec on_match(any, any, (any -&gt; any)) :: any @doc &quot;&quot;&quot; Given a pattern, it will return a function from expression to boolean. This function returns true if the expression matches the pattern and false otherwise. ## Examples: iex&gt; (match_pred({:good, _})).({:good, 1}) true iex&gt; (match_pred({:good, _})).({:bad, 1}) false &quot;&quot;&quot; @spec match_pred(any) :: (any -&gt; boolean) @doc &quot;&quot;&quot; Partitions an enum of tuples by their first element. Returns a map where the keys are the first elements of the tuples and the values are enums of the second elements that correspond. No guarantee on the order of the returned lists. (In fact they are generally backwards.) ## Example: iex&gt; [{:ok, 1}, {:error, 2}, {:error, 3}, {:ok, 4}, {:other, 1}] ...&gt; |&gt; partition %{ok: [4, 1], error: [3, 2], other: [1]} &quot;&quot;&quot; @spec partition(Enum.t({atom, any})) :: %{required(atom) =&gt; Enum.t()} Changed: s/~&gt;&gt;/~&gt;/g and eliminated nested capture problem split Base, Helpers, and Mappers into different files created a using macro to import entire library"},{"ref":"changelog.html#v0-3-0-03-25-19","title":"Changelog - v0.3.0 03/25/19","module":"changelog","type":"extras","doc":"Added: @doc &quot;&quot;&quot; Wraps a naked `:error` atom in a tuple with the given reason. Leaves success values and `error` tuples unchanged. ## Examples: iex&gt; :error ...&gt; |&gt; normalize_error(:parsing_failure) {:error, :parsing_failure} iex&gt; {:ok, 2} ...&gt; |&gt; normalize_error() {:ok, 2} &quot;&quot;&quot; @spec normalize_error(any, any) :: t() Removed: @doc &quot;&quot;&quot; Returns :ok Will be inlined at compile time. &quot;&quot;&quot; @spec ok() :: :ok @doc &quot;&quot;&quot; If the first argument is success then it returns the second, else it propogates the first error. Lazy. Only evaluates the second argument if necessary. ## Examples: iex&gt; sequencing({:ok, 1}, {:ok, 2}) {:ok, 2} iex&gt; sequencing({:error, 3}, {:error, 4}) {:error, 3} iex&gt; sequencing({:ok, 1}, :ok) :ok &quot;&quot;&quot; @spec sequencing(t(a), t(b)) :: t(b) @doc &quot;&quot;&quot; Infix sequencing. Lazy. Only evaluates the second argument if necessary. ## Examples: iex&gt; {:ok, 1} ...&gt; ~&gt; {:ok, 2} ...&gt; ~&gt; {:error, 3} ...&gt; ~&gt; {:error, 4} {:error, 3} iex&gt; {:ok, 1} ...&gt; ~&gt; {:ok, 2} ...&gt; ~&gt; {:ok, 3} {:ok, 3} &quot;&quot;&quot; @spec t(a) ~&gt; t(b) :: t(b) @doc &quot;Returns true if the given argument is an error tuple.&quot; @spec is_error(t(a)) :: boolean @doc &quot;Returns true if the given argument is :ok, or an ok tuple.&quot; @spec is_ok(t(a)) :: boolean @doc &quot;Flips an ok tuple to an error tuple and an error tuple to an ok tuple.&quot; @spec flip(s(a)) :: s(b) @doc &quot;&quot;&quot; Given an enum of plain values, an initial value, and a reducing function, it returns the first `error` or reduced result. ## Examples: iex&gt; [{:ok, 1}, {:error, 2}, {:error, 3}, {:ok, 4}] ...&gt; |&gt; sequence {:error, 2} iex&gt; [{:ok, 1}, {:ok, 2}, {:ok, 3}, {:ok, 4}] ...&gt; |&gt; sequence {:ok, [1, 2, 3, 4]} &quot;&quot;&quot; @spec sequence(Enum.t(s(a))) :: s(Enum.t(a)) @doc &quot;&quot;&quot; Given an enum of tuples, an initial value, and a reducing function, it returns the first error or reduced result. ## Examples: iex&gt; [{:ok, 1}, {:ok, 2}, {:ok, 3}, {:ok, 4}] ...&gt; |&gt; reduce_unless_error({:ok, 100}, &amp;{:ok, &amp;1 + &amp;2}) {:ok, 110} @spec reduce_unless_error(Enum.t(s(a)), s(a), ((a, b) -&gt; s(b))) :: s(b) @doc &quot;&quot;&quot; Given an enum of tuples and a reducing function, it returns the first error or reduced result. Does not require an initial result, instead it uses the first element of the enum. Raises an error on empty list and simply returns the value on singletons. ## Examples: iex&gt; [{:ok, 1}, {:ok, 2}, {:ok, 3}, {:ok, 4}] ...&gt; |&gt; reduce_unless_error(&amp;{:ok, &amp;1 + &amp;2}) {:ok, 10} iex&gt; [{:ok, 1}, {:error, 2}, {:ok, 3}, {:error, 4}] ...&gt; |&gt; reduce_unless_error(&amp;{:ok, &amp;1 + &amp;2}) {:error, 2} &quot;&quot;&quot; @spec reduce_unless_error(Enum.t(s(a)), ((a, b) -&gt; s(b))) :: s(b) | s(a) @doc &quot;&quot;&quot; Returns first error or returns last element in enum. ## Examples: iex&gt; [{:ok, 1}, {:error, 3}, {:ok, 2}, {:error, 4}] ...&gt; |&gt; stop_at_first_error {:error, 3} iex&gt; [{:ok, 1}, {:ok, 2}, {:ok, 3}, {:ok, 4}] ...&gt; |&gt; stop_at_first_error {:ok, 4} &quot;&quot;&quot; @spec stop_at_first_error(Enum.t(t(a))) :: t(a) @doc &quot;&quot;&quot; Returns an enum of values that were successful and passed the filtering function. ## Examples: iex&gt; [{:ok, 1}, {:error, 2}, {:error, 3}, {:ok, 4}] ...&gt; |&gt; filter(fn x -&gt; x &lt; 3 end) {:ok, [1]} iex&gt; [{:ok, 1}, {:error, 2}, {:error, 3}, {:ok, 4}] ...&gt; |&gt; filter(fn _ -&gt; true end) {:ok, [1, 4]} &quot;&quot;&quot; @spec filter(Enum.t(s(a)), (a -&gt; boolean)) :: t(Enum.t(a)) Other Changes: s/extract/extract!/g @doc &quot;&quot;&quot; Extracts the value or reason from the tuple. Caution: If given an `error` tuple it raise an exception! &quot;&quot;&quot; @spec extract!(s(a)) :: a @doc &quot;Extracts the value or reason from the tuple.&quot; @spec extract(s(a)) :: a Enforced return type on bind/2"},{"ref":"changelog.html#v0-2-0-12-11-18","title":"Changelog - v0.2.0 12/11/18","module":"changelog","type":"extras","doc":"Added: @doc &quot;&quot;&quot; Ignores the value in an ok tuple and just returns :ok. Still shortcircuits on error. ## Examples: iex&gt; {:ok, 2} ...&gt; |&gt; ignore :ok iex&gt; :ok ...&gt; |&gt; ignore :ok iex&gt; {:error, :not_found} ...&gt; |&gt; ignore {:error, :not_found} &quot;&quot;&quot; @spec ignore(t(a)) :: p() def ignore(ma), do: sequencing(ma, :ok) @doc &quot;&quot;&quot; Applies the function to each element of the enumerable until an error occurs. No guarentee on the order of evaluation. (But usually backwards for lists.) Only takes a function that returns :ok | {:error, value}. ## Examples: iex&gt; [1, 2, 3, 4] ...&gt; |&gt; each_while_success(fn x -&gt; if x &lt; 3, do: :ok, else: {:error, :too_big} end) {:error, :too_big} iex&gt; [1, 2, 3, 4] ...&gt; |&gt; each_while_success(fn x -&gt; if x &lt; 5, do: :ok, else: {:error, :too_big} end) :ok &quot;&quot;&quot; @spec each_while_success(Enum.t(a), (a -&gt; p())) :: p() Other Changes: Added laziness to: @doc &quot;&quot;&quot; If the first argument is success then it returns the second, else it propogates the first error. Lazy. Only evaluates the second argument if necessary. ## Examples: iex&gt; sequencing({:ok, 1}, {:ok, 2}) {:ok, 2} iex&gt; sequencing({:error, 3}, {:error, 4}) {:error, 3} iex&gt; sequencing({:ok, 1}, :ok) :ok iex&gt; sequencing(:ok, {:ok, 1}) {:ok, 1} &quot;&quot;&quot; @spec sequencing(t(a), t(b)) :: t(b) @doc &quot;&quot;&quot; Infix sequencing. Lazy. Only evaluates the second argument if necessary. ## Examples: iex&gt; {:ok, 1} ...&gt; ~&gt; {:ok, 2} ...&gt; ~&gt; {:error, 3} ...&gt; ~&gt; {:error, 4} {:error, 3} iex&gt; {:ok, 1} ...&gt; ~&gt; {:ok, 2} ...&gt; ~&gt; {:ok, 3} {:ok, 3} iex&gt; {:ok, 1} ...&gt; ~&gt; :ok :ok &quot;&quot;&quot; @spec t(a) ~&gt; t(b) :: t(b) @doc &quot;&quot;&quot; Replaces the reason in an error tuple. Leaves success unchanged. Lazy. Only evaluates the second argument if necessary. ## Example: iex&gt; account_name = &quot;test&quot; ...&gt; {:error, :not_found} ...&gt; |&gt; mask_error({:nonexistent_account, account_name}) {:error, {:nonexistent_account, &quot;test&quot;}} &quot;&quot;&quot; @spec mask_error(t(a), any) :: t(a) @doc &quot;&quot;&quot; Converts a matching error to a success with the given value or function. An error matches if the reason is equal to the supplied atom or the reason passes the predicate. Leaves success and other errors unchanged. Lazy. Only evaluates the second argument if necessary. ## Example: iex&gt; {:error, :already_completed} ...&gt; |&gt; convert_error(:already_completed, &quot;submitted&quot;) {:ok, &quot;submitted&quot;} iex&gt; {:error, &quot;test&quot;} ...&gt; |&gt; convert_error(&amp;(&amp;1 == &quot;test&quot;), fn r -&gt; {:ok, r &lt;&gt; &quot;ed&quot;} end) {:ok, &quot;tested&quot;} &quot;&quot;&quot; @spec convert_error(t(a), (any -&gt; boolean) | any, b | (any -&gt; t(b))) :: t(b) @doc &quot;&quot;&quot; Lifts a value into a success tuple unless: 1) the value matches the second argument 2) when applied to the value, the second argument function returns true In those cases an error tuple is returned with either 1) the third argument as the reason 2) the third argument function applied to the value, as the reason lift is lazy, this means third argument will only be evaluated when necessary. ## Examples: iex&gt; nil ...&gt; |&gt; lift(nil, :not_found) {:error, :not_found} iex&gt; 2 ...&gt; |&gt; lift(nil, :not_found) {:ok, 2} iex&gt; &quot;test&quot; ...&gt; |&gt; lift(&amp;(&amp;1 == &quot;test&quot;), fn x -&gt; {:oops, x &lt;&gt; &quot;ed&quot;} end) {:error, {:oops, &quot;tested&quot;}} &quot;&quot;&quot; @spec lift(a | b, b | (a | b -&gt; boolean), c | (a | b -&gt; c)) :: s(a)"},{"ref":"changelog.html#v0-1-0-11-30-18","title":"Changelog - v0.1.0 11/30/18","module":"changelog","type":"extras","doc":"Initial version!"},{"ref":"releasing.html","title":"Releasing a New Version","module":"releasing","type":"extras","doc":"Releasing a New Version"},{"ref":"releasing.html#checklist","title":"Releasing a New Version - Checklist","module":"releasing","type":"extras","doc":"Ensure CHANGELOG.md is up-to-date Ensure working dir is clean Run mix format and mix credo for formatting and linting Ensure the docs look acceptable using mix docs Run prettier on all markdown files Update version requirement in README.md Update version in mix.exs Create a commit: git commit -a -m &quot;Bump version to 0.X.Y&quot; git tag v0.X.Y mix compile --warnings-as-errors &amp;&amp; mix test &amp;&amp; mix hex.publish git push origin master --tags Enjoy!"}]